Module: Dentaku::Error
- Included in:
- ArgumentError, BaseError, ZeroDivisionError
- Defined in:
- lib/dentaku/exceptions.rb
Overview
Shared by all Dentaku exceptions so that rescue Dentaku::Error catches
everything the gem raises, including exceptions that subclass Ruby
built-ins (Dentaku::ArgumentError, Dentaku::ZeroDivisionError).
Instance Attribute Summary collapse
-
#assigned_to ⇒ Object
Returns the value of attribute assigned_to.
Instance Attribute Details
#assigned_to ⇒ Object
Returns the value of attribute assigned_to.
6 7 8 |
# File 'lib/dentaku/exceptions.rb', line 6 def assigned_to @assigned_to end |